/* SPDX-License-Identifier: GPL-2.0 */
/* Copyright (c) 2018, Linaro Ltd. */

#ifndef __DT_BINDINGS_POWER_QCOM_AOSS_QMP_H
#define __DT_BINDINGS_POWER_QCOM_AOSS_QMP_H

#define AOSS_QMP_LS_CDSP		0
#define AOSS_QMP_LS_LPASS	1
#define AOSS_QMP_LS_MODEM	2
#define AOSS_QMP_LS_SLPI		3
#define AOSS_QMP_LS_SPSS		4
#define AOSS_QMP_LS_VENUS	5

#endif
