/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * linux/include/asm-generic/ftrace.h
 */
#ifndef __ASM_GENERIC_FTRACE_H__
#define __ASM_GENERIC_FTRACE_H__

/*
 * Not all architectures need their own ftrace.h, the most
 * common definitions are already in linux/ftrace.h.
 */

#endif /* __ASM_GENERIC_FTRACE_H__ */
