/* SPDX-License-Identifier: GPL-2.0 */

#ifndef _DT_BINDINGS_CLK_LPASS_AUDIOCC_SM8250_H
#define _DT_BINDINGS_CLK_LPASS_AUDIOCC_SM8250_H

/* From AudioCC */
#define LPASS_CDC_WSA_NPL				0
#define LPASS_CDC_WSA_MCLK				1
#define LPASS_CDC_RX_MCLK				2
#define LPASS_CDC_RX_NPL				3
#define LPASS_CDC_RX_MCLK_MCLK2				4

#endif /* _DT_BINDINGS_CLK_LPASS_AUDIOCC_SM8250_H */
