/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_BINDINGS_POWER_RK3228_POWER_H__
#define __DT_BINDINGS_POWER_RK3228_POWER_H__

/**
 * RK3228 idle id Summary.
 */

#define RK3228_PD_CORE		0
#define RK3228_PD_MSCH		1
#define RK3228_PD_BUS		2
#define RK3228_PD_SYS		3
#define RK3228_PD_VIO		4
#define RK3228_PD_VOP		5
#define RK3228_PD_VPU		6
#define RK3228_PD_RKVDEC	7
#define RK3228_PD_GPU		8
#define RK3228_PD_PERI		9
#define RK3228_PD_GMAC		10

#endif
